import { APIClient } from "./ApiClient";

export type Sex = "M" | "F";

export interface MemberDataApi {
  id: number;
  family_id: number;
  first_name?: string;
  last_name?: string;
  birth_last_name?: string;
  photo_id?: number;
  signed_photo_id?: number;
  email?: string;
  phone?: string;
  address?: string;
  city?: string;
  postal_code?: string;
  country?: string;
  sex?: Sex;
  time_create?: number;
  time_update?: number;
  mother?: number;
  father?: number;
  birth_year?: number;
  birth_month?: number;
  birth_day?: number;
  dead: boolean;
  death_year?: number;
  death_month?: number;
  death_day?: number;
  note?: string;
}

export class Member implements MemberDataApi {
  id: number;
  family_id: number;
  first_name?: string;
  last_name?: string;
  birth_last_name?: string;
  photo_id?: number;
  signed_photo_id?: number;
  email?: string;
  phone?: string;
  address?: string;
  city?: string;
  postal_code?: string;
  country?: string;
  sex?: Sex;
  time_create?: number;
  time_update?: number;
  mother?: number;
  father?: number;
  birth_year?: number;
  birth_month?: number;
  birth_day?: number;
  dead!: boolean;
  death_year?: number;
  death_month?: number;
  death_day?: number;
  note?: string;

  constructor(m: MemberDataApi) {
    this.id = m.id;
    this.family_id = m.family_id;
    this.first_name = m.first_name;
    this.last_name = m.last_name;
    this.birth_last_name = m.birth_last_name;
    this.photo_id = m.photo_id;
    this.signed_photo_id = m.signed_photo_id;
    this.email = m.email;
    this.phone = m.phone;
    this.address = m.address;
    this.city = m.city;
    this.postal_code = m.postal_code;
    this.country = m.country;
    this.sex = m.sex;
    this.time_create = m.time_create;
    this.time_update = m.time_update;
    this.mother = m.mother;
    this.father = m.father;
    this.birth_year = m.birth_year;
    this.birth_month = m.birth_month;
    this.birth_day = m.birth_day;
    this.dead = m.dead;
    this.death_year = m.death_year;
    this.death_month = m.death_month;
    this.death_day = m.death_day;
    this.note = m.note;
  }

  /**
   * Create an empty member object
   */
  static New(family_id: number): Member {
    return new Member({
      id: 0,
      dead: false,
      family_id: family_id,
    });
  }

  get fullName(): string {
    const firstName = this.first_name ?? "";

    return firstName.length === 0
      ? this.last_name ?? ""
      : `${firstName} ${this.last_name ?? ""}`;
  }
}

export class MembersList {
  private list: Member[];
  private map: Map<number, Member>;

  constructor(list: Member[]) {
    this.list = list;
    this.map = new Map();

    for (const m of list) {
      this.map.set(m.id, m);
    }
  }
}

export class MemberApi {
  /**
   * Create a new member
   */
  static async Create(m: Member): Promise<Member> {
    const res = await APIClient.exec({
      uri: `/family/${m.family_id}/member/create`,
      method: "POST",
      jsonData: m,
    });

    return new Member(res.data);
  }

  /**
   * Get the information about a single member
   */
  static async GetSingle(
    family_id: number,
    member_id: number
  ): Promise<Member> {
    const res = await APIClient.exec({
      uri: `/family/${family_id}/member/${member_id}`,
      method: "GET",
    });

    return new Member(res.data);
  }

  /**
   * Get the entire list of family members of a family
   */
  static async GetEntireList(family_id: number): Promise<MembersList> {
    const res = await APIClient.exec({
      uri: `/family/${family_id}/members`,
      method: "GET",
    });

    return new MembersList(res.data.map((d: any) => new Member(d)));
  }

  /**
   * Update a member information
   */
  static async Update(m: Member): Promise<void> {
    await APIClient.exec({
      uri: `/family/${m.family_id}/member/${m.id}`,
      method: "PUT",
      jsonData: m,
    });
  }

  /**
   * Delete a family member
   */
  static async Delete(m: Member): Promise<void> {
    await APIClient.exec({
      uri: `/family/${m.family_id}/member/${m.id}`,
      method: "DELETE",
    });
  }
}
